@@ -1,6 +1,6 @@
 ;;; mime-w3m.el --- mime-view content filter for text
 
-;; Copyright (C) 2001,2002 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001, 2002 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Akihiro Arisawa    <ari@mbf.sphere.ne.jp>
@@ -41,7 +41,7 @@
   (require 'mime)
   (defvar mime-setup-enable-inline-html))
 
-(defcustom mime-w3m-display-inline-image 'default
+(defcustom mime-w3m-display-inline-images 'default
   "Non-nil means that inline images are displayed."
   :group 'w3m
   :group 'mime-view
@@ -94,6 +94,7 @@ consider all the urls to be safe."
     (w3m-bookmark-view)
     (w3m-close-window)
     (w3m-copy-buffer)
+    (w3m-delete-buffer)
     (w3m-dtree)
     (w3m-edit-current-url)
     (w3m-edit-this-url)
@@ -105,12 +106,16 @@ consider all the urls to be safe."
     (w3m-history-store-position)
     (w3m-mouse-view-this-url . mime-w3m-mouse-view-this-url)
     (w3m-namazu)
+    (w3m-next-buffer)
+    (w3m-previous-buffer)
     (w3m-quit)
     (w3m-redisplay-with-charset)
     (w3m-reload-this-page)
     (w3m-scroll-down-or-previous-url)
     (w3m-scroll-up-or-next-url)
     (w3m-search)
+    (w3m-select-buffer)
+    (w3m-switch-buffer)
     (w3m-view-header)
     (w3m-view-parent-page)
     (w3m-view-previous-page)
@@ -131,8 +136,8 @@ will not be substituted.")
 (defsubst mime-w3m-setup ()
   "Setup `mime-w3m' module."
   (require 'w3m)
-  (when (eq mime-w3m-display-inline-image 'default)
-    (setq mime-w3m-display-inline-image w3m-display-inline-image))
+  (when (eq mime-w3m-display-inline-images 'default)
+    (setq mime-w3m-display-inline-images w3m-default-display-inline-images))
   (unless (assq 'mime-view-mode w3m-cid-retrieve-function-alist)
     (push (cons 'mime-view-mode 'mime-w3m-cid-retrieve)
 	  w3m-cid-retrieve-function-alist))
@@ -178,7 +183,7 @@ will not be substituted.")
        (run-hooks 'mime-text-decode-hook)
        (condition-case err
 	   (let ((w3m-safe-url-regexp mime-w3m-safe-url-regexp)
-		 (w3m-current-image-status mime-w3m-display-inline-image)
+		 (w3m-display-inline-images mime-w3m-display-inline-images)
 		 w3m-force-redisplay)
 	     (w3m-region p
 			 (point-max)
@@ -199,12 +204,12 @@ will not be substituted.")
 
 (defun mime-w3m-cid-retrieve (url &rest args)
   (let ((entity (mime-find-entity-from-content-id
-		 (mime-uri-parse-cid url) mime-w3m-message-structure)))
+		 (mime-uri-parse-cid url)
+		 (with-current-buffer w3m-current-buffer
+		   mime-w3m-message-structure))))
     (when entity
-      (w3m-with-work-buffer
-       (delete-region (point-min) (point-max))
-       (set-buffer-multibyte nil)
-       (mime-insert-entity-content entity))
+      ;; `mime-decode-string' should be performed in a unibyte buffer.
+      (w3m-insert-string (mime-entity-content entity))
       (mime-entity-type/subtype entity))))
 
 (defun mime-w3m-view-this-url ()
