@@ -1,10 +1,11 @@
 ;;; sb-asahi-html.el --- shimbun backend for asahi.com (HTML version)
 
-;; Author: Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001 Yuuichi Teranishi  <teranisi@gohome.org>
 
+;; Author: Yuuichi Teranishi  <teranisi@gohome.org>
 ;; Keywords: news
 
-;;; Copyright:
+;; This file is a part of shimbun.
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -30,7 +31,7 @@
 
 (luna-define-class shimbun-asahi-html (shimbun shimbun-asahi) ())
 
-(defvar shimbun-asahi-html-url "http://spin.asahi.com/")
+(defvar shimbun-asahi-html-url "http://www.asahi.com/")
 (defvar shimbun-asahi-html-groups '("national" "business" "politics"
 				    "international" "sports" "culture"))
 (defvar shimbun-asahi-html-from-address "webmaster@www.asahi.com")
