@@ -37,6 +37,7 @@
 
 ;;; Code:
 
+(require 'w3m-util)
 (require 'w3m)
 
 (eval-when-compile
@@ -56,7 +57,7 @@
 
 ;;; w3m-form structure:
 
-(defun w3m-form-new (method action &optional baseurl charlst)
+(defun w3m-form-new (method action &optional baseurl charlst enctype)
   "Return new form object."
   (vector 'w3m-form-object
 	  (if (stringp method)
@@ -65,6 +66,7 @@
 	  (and action
 	       (w3m-expand-url action baseurl))
 	  charlst
+	  (or enctype 'urlencoded)
 	  nil))
 
 (defsubst w3m-form-p (obj)
@@ -79,16 +81,25 @@
   (` (aref (, form) 2)))
 (defmacro w3m-form-charlst (form)
   (` (aref (, form) 3)))
-(defmacro w3m-form-plist (form)
+(defmacro w3m-form-enctype (form)
   (` (aref (, form) 4)))
+(defmacro w3m-form-plist (form)
+  (` (aref (, form) 5)))
+(defsubst w3m-form-put-property (form name property value)
+  (aset form 5
+	(plist-put (w3m-form-plist form)
+		   (setq name (intern name))
+		   (plist-put (plist-get (w3m-form-plist form) name)
+			      property value)))
+  value)
+(defmacro w3m-form-get-property (form name property)
+  (` (plist-get (plist-get (w3m-form-plist (, form))
+			   (intern (, name)))
+		(, property))))
 (defmacro w3m-form-put (form name value)
-  (let ((tempvar (make-symbol "formobj")))
-    (` (let (((, tempvar) (, form)))
-	 (aset (, tempvar) 4
-	       (plist-put (w3m-form-plist (, tempvar))
-			  (intern (, name)) (, value)))))))
+  (` (w3m-form-put-property (, form) (, name) :value (, value))))
 (defmacro w3m-form-get (form name)
-  (` (plist-get (w3m-form-plist (, form)) (intern (, name)))))
+  (` (w3m-form-get-property (, form) (, name) :value)))
 
 (defun w3m-form-goto-next-field ()
   "Move to next form field and return the point.
@@ -102,10 +113,10 @@ If no field in forward, return nil without moving."
 	(goto-char next)
       nil)))
 
-(defun w3m-form-make-form-data (form &optional urlencode)
+(defun w3m-form-make-form-data (form)
   (let ((plist (w3m-form-plist form))
 	(coding (w3m-form-charlst form))
-	buf multipart)
+	buf)
     (setq coding
 	  (or (catch 'det
 		(while coding
@@ -118,11 +129,10 @@ If no field in forward, return nil without moving."
 	      w3m-default-coding-system))
     (while plist
       (let ((name (symbol-name (car plist)))
-	    (value (cadr plist)))
+	    (value (plist-get (cadr plist) :value)))
 	(cond
 	 ((and (consp value)
 	       (eq (car value) 'file))
-	  (setq multipart t)
 	  (setq buf (cons (cons name value) buf)))
 	 ((and (consp value)
 	       (consp (cdr value))
@@ -135,8 +145,7 @@ If no field in forward, return nil without moving."
 	  (setq buf (cons (cons name value) buf))))
 	(setq plist (cddr plist))))
     (when buf
-      (if (and multipart
-	       (not urlencode))
+      (if (eq (w3m-form-enctype form) 'multipart)
 	  (let ((boundary (apply 'format "--_%d_%d_%d" (current-time)))
 		file type)
 	    (setq buf (nreverse buf))
@@ -179,241 +188,93 @@ If no field in forward, return nil without moving."
 						    coding)))
 		   buf "&")))))
 
-;;;###autoload
-(defun w3m-form-parse-buffer ()
-  "Parse HTML data in this buffer and return form/map objects.
-Check the cached form/map objects are available, and if available
-return them with the flag."
-  (or (when (w3m-cache-available-p w3m-current-url)
-	(let ((forms (w3m-history-plist-get :forms w3m-current-url nil t)))
-	  ;; Mark that `w3m-current-forms' is resumed from history.
-	  (and forms (cons t forms))))
-      (nreverse
-       (w3m-form-parse-forms))))
-
-(defun w3m-form-parse-forms ()
-  "Parse Form/usemap objects in this buffer."
-  (let ((case-fold-search t)
-	forms tag)
-    (goto-char (point-min))
-    (while (re-search-forward (w3m-tag-regexp-of "form" "img") nil t)
-      (setq tag (downcase (match-string 1)))
-      (goto-char (match-end 1))
-      (cond
-       ((string= tag "img")
-	;; Parse USEMAP property of IMG tag
-	(w3m-parse-attributes (usemap)
-	  (when usemap
-	    (if (not (string-match "^#" usemap))
-		(setq forms (cons nil forms)) ;; Sure ?
-	      (setq usemap (substring usemap 1))
-	      (setq forms
-		    (cons (w3m-form-new "map" usemap)
-			  forms))
-	      (save-excursion
-		(goto-char (point-min))
-		(let (candidates)
-		  (when (re-search-forward
-			 (concat "<map +name=\"" usemap "\"[^>]*>") nil t)
-		    (while (and (re-search-forward
-				 (w3m-tag-regexp-of "area" "/map") nil t)
-				(not (char-equal
-				      (char-after (match-beginning 1))
-				      ?/)))
-		      (goto-char (match-end 1))
-		      (w3m-parse-attributes (href alt)
-			(when href
-			  (setq candidates (cons (cons href (or alt href))
-						 candidates)))))
-		    (when candidates
-		      (w3m-form-put (car forms)
-				    "link"
-				    (nreverse candidates))))))))))
-       (t
-	;; Parse attribute of FORM tag
-	;; accept-charset <= charset,charset,...
-	;; charset <= valid only w3mmee with frame
-	(w3m-parse-attributes (action (method :case-ignore)
-				      (accept-charset :case-ignore)
-				      (charset :case-ignore))
-	  (if accept-charset
-	      (setq accept-charset (split-string accept-charset ","))
-	    (when (and charset (eq w3m-type 'w3mmee))
-	      (cond
-	       ((string= charset "e")	;; w3mee without libmoe
-		(setq accept-charset (list "euc-jp")))
-	       ((string= charset "s")	;; w3mee without libmoe
-		(setq accept-charset (list "shift-jis")))
-	       ((string= charset "n")	;; w3mee without libmoe
-		(setq accept-charset (list "iso-2022-7bit")))
-	       (t				;; w3mee with libmoe
-		(setq accept-charset (list charset))))))
-	  (setq forms
-		(cons (w3m-form-new
-		       (or method "get")
-		       (or action
-			   (and w3m-current-url
-				(string-match w3m-url-components-regexp
-					      w3m-current-url)
-				(substring w3m-current-url 0
-					   (or (match-beginning 6)
-					       (match-beginning 8)))))
-		       nil
-		       accept-charset)
-		      forms)))
-	;; Parse form fields until </FORM>
-	(while (and (re-search-forward
-		     (w3m-tag-regexp-of "input" "textarea" "select" "/form")
-		     nil t)
-		    (not (char-equal (char-after (match-beginning 1)) ?/)))
-	  (setq tag (downcase (match-string 1)))
-	  (goto-char (match-end 1))	; go to end of tag name
-	  (cond
-	   ((string= tag "input")
-	    ;; When <INPUT> is found.
-	    (w3m-parse-attributes (name (value :decode-entity)
-					(type :case-ignore) (checked :bool))
-	      (when name
-		(cond
-		 ((or (string= type "submit")
-		      (string= type "image"))
-		  ;; Submit button input, not set name and value here.
-		  ;; They are set in `w3m-form-submit'.
-		  nil)
-		 ((string= type "checkbox")
-		  ;; Check box input, one name has multiple values
-		  ;; Value is list of item VALUE which has same NAME.
-		  (let ((cvalue (w3m-form-get (car forms) name)))
-		    (w3m-form-put (car forms) name
-				  (if checked
-				      (cons value cvalue)
-				    cvalue))))
-		 ((string= type "radio")
-		  ;; Radio button input, one name has one value
-		  (w3m-form-put (car forms) name
-				(if checked value
-				  (w3m-form-get (car forms) name))))
-		 (t
-		  ;; ordinaly text input
-		  (w3m-form-put (car forms)
-				name
-				(or value (w3m-form-get (car forms)
-							name))))))))
-	   ((string= tag "textarea")
-	    ;; When <TEXTAREA> is found.
-	    (w3m-parse-attributes (name)
-	      (let ((start (point))
-		    value)
-		(skip-chars-forward "^<")
-		(setq value (w3m-decode-entities-string
-			     (buffer-substring start (point))))
-		(when name
-		  (w3m-form-put (car forms)
-				name
-				(or value (w3m-form-get (car forms) name)))))))
-	   ;; When <SELECT> is found.
-	   ((string= tag "select")
-	    (let (vbeg svalue cvalue candidates)
-	      (goto-char (match-end 1))
-	      (w3m-parse-attributes (name)
-		;; Parse FORM SELECT fields until </SELECT> (or </FORM>)
-		(while (and (re-search-forward
-			     (w3m-tag-regexp-of "option" "/select" "/form")
-			     nil t)
-			    (not (char-equal (char-after (match-beginning 1))
-					     ?/)))
-		  ;; <OPTION> is found
-		  (goto-char (match-end 1)) ; goto very after "<xxxx"
-		  (w3m-parse-attributes (value (selected :bool))
-		    (setq vbeg (point))
-		    (skip-chars-forward "^<")
-		    (setq svalue
-			  (w3m-decode-entities-string
-			   (mapconcat 'identity
-				      (split-string
-				       (buffer-substring vbeg (point))
-				       "[ \t\r\f\n]+")
-				      " ")))
-		    (unless value
-		      (setq value svalue))
-		    (when selected
-		      (setq cvalue value))
-		    (push (cons value svalue) candidates)))
-		(when name
-		  (w3m-form-put (car forms) name (cons
-						  cvalue ; current value
-						  (nreverse
-						   candidates))))))))))))
-    forms))
-
 (defun w3m-form-resume (forms)
   "Resume content of all forms in the current buffer using FORMS."
+  (if (eq (car forms) t)
+      (setq forms (cdr forms)))
   (save-excursion
     (goto-char (point-min))
-    (let (fid type name form textareas)
+    (let (fid type name form cform textareas)
       (while (w3m-form-goto-next-field)
 	(setq fid (get-text-property (point) 'w3m-form-field-id))
 	(when (and fid
 		   (string-match
 		    "fid=\\([^/]+\\)/type=\\([^/]+\\)/name=\\(.*\\)$"
-		    fid)
-		   (setq type (match-string 2 fid)
-			 name (match-string 3 fid)
-			 form (nth (string-to-number (match-string 1 fid))
-				   forms)))
+		    fid))
+	  (setq form (nth (string-to-number (match-string 1 fid))
+			  forms)
+		cform (nth (string-to-number (match-string 1 fid))
+			   w3m-current-forms)
+		type (match-string 2 fid)
+		name (match-string 3 fid))
 	  (cond
 	   ((or (string= type "submit")
 		(string= type "image"))
 	    ;; Remove status to support forms containing multiple
 	    ;; submit buttons.
-	    (w3m-form-put form name nil))
+	    (w3m-form-put cform name nil))
 	   ((or (string= type "reset")
 		(string= type "hidden")
 		;; Do nothing.
 		))
 	   ((string= type "password")
 	    (w3m-form-replace (w3m-form-get form name)
-			      'invisible))
-	   ((or (string= type "checkbox")
-		(string= type "radio"))
-	    (when (stringp (w3m-form-get form name))
-	      (w3m-form-replace
-	       (if (string= (w3m-form-get form name)
-			    (nth 3 (w3m-action)))
-		   "*" " "))))
+			      'invisible)
+	    (unless (eq form cform)
+	      (w3m-form-put cform name (w3m-form-get form name))))
+	   ((string= type "radio")
+	    (let ((value (w3m-form-get form name)))
+	      (when value
+		(w3m-form-replace
+		 (if (string= value (nth 3 (w3m-action)))
+		     "*" " ")))
+	      (unless (eq form cform)
+		(w3m-form-put cform name value))))
+	   ((string= type "checkbox")
+	    (let ((value (w3m-form-get form name)))
+	      (when value
+		(w3m-form-replace
+		 (if (member (nth 3 (w3m-action)) value)
+		     "*" " ")))
+	      (unless (eq form cform)
+		(w3m-form-put cform name value))))
 	   ((string= type "select")
 	    (let ((selects (w3m-form-get form name)))
 	      (when (car selects)
-		(w3m-form-replace (cdr (assoc (car selects) (cdr selects)))))))
+		(w3m-form-replace (cdr (assoc (car selects) (cdr selects)))))
+	      (unless (eq form cform)
+		(w3m-form-put cform name selects))))
 	   ((string= type "textarea")
-	    (let ((hseq (nth 2 (w3m-action))))
+	    (let ((hseq (nth 2 (w3m-action)))
+		  (value (w3m-form-get form name)))
 	      (when (> hseq 0)
-		(setq textareas
-		      (cons (cons hseq (w3m-form-get form name))
-			    textareas)))))
+		(setq textareas (cons (cons hseq value) textareas)))
+	      (unless (eq form cform)
+		(w3m-form-put cform name value))))
 	   ((string= type "file")
 	    (let ((value (w3m-form-get form name)))
 	      (when (and value
 			 (consp value))
-		(w3m-form-replace (cdr value)))))
+		(w3m-form-replace (cdr value)))
+	      (unless (eq form cform)
+		(w3m-form-put cform name value))))
 	   (t
 	    (let ((value (w3m-form-get form name)))
 	      (when value
-		(w3m-form-replace value)))))))
+		(w3m-form-replace value))
+	      (unless (eq form cform)
+		(w3m-form-put cform name value)))))))
       (dolist (textarea textareas)
 	(when (cdr textarea)
 	  (w3m-form-textarea-replace (car textarea) (cdr textarea)))))))
 
 ;;;###autoload
 (defun w3m-fontify-forms ()
-  "Process half-dumped data in this buffer and fontify <input_alt> tags."
-  ;; Check whether `w3m-current-forms' is resumed from history.
-  (if (eq t (car w3m-current-forms))
-      (progn
-	(setq w3m-current-forms (cdr w3m-current-forms))
-	(w3m-form-fontify w3m-current-forms)
-	(w3m-form-resume w3m-current-forms))
-    (w3m-form-fontify w3m-current-forms)))
+  "Process half-dumped data and fontify forms in this buffer."
+  ;; If `w3m-current-forms' is resumed from history, reuse it.
+  (w3m-form-parse-and-fontify
+   (when (eq t (car w3m-current-forms))
+     (setq w3m-current-forms (cdr w3m-current-forms)))))
 
 (eval-and-compile
   (unless (fboundp 'w3m-form-make-button)
@@ -421,124 +282,358 @@ return them with the flag."
       "Make button on the region from START to END."
       (add-text-properties start end (append '(face w3m-form-face) properties)))))
 
-(defun w3m-form-fontify (forms)
-  "Process half-dumped data in this buffer and fontify <input_alt> tags using FORMS."
-  (goto-char (point-min))
-  (while (search-forward "<input_alt " nil t)
-    (let (start)
-      (setq start (match-beginning 0))
-      (goto-char (match-end 0))
-      (w3m-parse-attributes ((fid :integer)
-			     (type :case-ignore)
-			     (width :integer)
-			     (maxlength :integer)
-			     (hseq :integer)
-			     name value)
-	(delete-region start (point))
-	(search-forward "</input_alt>")
-	(goto-char (match-beginning 0))
-	(delete-region (match-beginning 0) (match-end 0))
-	(let ((form (nth fid forms)))
-	  (when form
-	    (cond
-	     ((and (string= type "hidden")
-		   (string= name "link"))
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-map ,form ,name)
-		 w3m-cursor-anchor (w3m-form-input-map ,form ,name))))
-	     ((or (string= type "submit")
-		  (string= type "image"))
-	      (w3m-form-make-button
-	       start (point)
-	       `(w3m-action (w3m-form-submit ,form ,name ,value)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-submit ,form))))
-	     ((string= type "reset")
-	      (w3m-form-make-button
-	       start (point)
-	       `(w3m-action (w3m-form-reset ,form)
-		 w3m-cursor-anchor (w3m-form-reset ,form))))
-	     ((string= type "textarea")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-textarea ,form ,hseq)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-form-hseq ,hseq))
-	      (when (> hseq 0)
-		(add-text-properties
-		 start (point)
-		 `(w3m-cursor-anchor
-		   (w3m-form-input-textarea ,form ,hseq)
-		   w3m-form-name ,name))))
-	     ((string= type "select")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-select ,form ,name)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input-select ,form ,name))))
-	     ((string= type "password")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-password ,form ,name)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input-password ,form ,name))))
-	     ((string= type "checkbox")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-checkbox ,form ,name ,value)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input-checkbox ,form ,name
-							    ,value))))
-	     ((string= type "radio")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-radio ,form ,name ,value)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input-radio ,form ,name ,value))))
-	     ((string= type "file")
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input-file ,form ,name ,value)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input-file ,form ,name ,value))))
-	     (t ;; input button.
-	      (add-text-properties
-	       start (point)
-	       `(face
-		 w3m-form-face
-		 w3m-action (w3m-form-input ,form ,name ,type
-					    ,width ,maxlength ,value)
-		 w3m-submit (w3m-form-submit ,form ,name
-					     (w3m-form-get ,form ,name))
-		 w3m-cursor-anchor (w3m-form-input ,form ,name ,type
-						   ,width ,maxlength
-						   ,value)))))))
-	(put-text-property start (point)
-			   'w3m-form-field-id
-			   (format "fid=%d/type=%s/name=%s" fid type name))))))
+;;; w3mmee
+;;
+(if (fboundp 'char-to-int)
+    (defalias 'w3m-char-to-int 'char-to-int)
+  (defalias 'w3m-char-to-int 'identity))
 
+(defmacro w3m-form-mee-attr-unquote (x)
+  "Unquote form attribute of w3mmee."
+  '(let (attr)
+     (when (eq (car x) ?T)
+       (setq x (cdr x))
+       (while (and x (not (eq (w3m-char-to-int (car x)) 0)))
+	 (setq attr (concat attr (char-to-string (car x))))
+	 (setq x (cdr x))))
+     attr))
+
+(if (fboundp 'string-to-list)
+    (defalias 'w3m-string-to-char-list 'string-to-list)
+  (defun w3m-string-to-char-list (str)
+    (mapcar 'identity str)))
+
+(if (fboundp 'int-to-char)
+    (defalias 'w3m-int-to-char 'int-to-char)
+  (defalias 'w3m-int-to-char 'identity))
+
+(defun w3m-form-mee-new (x)
+  "Decode form information of w3mmee."
+  (setq x (w3m-string-to-char-list (w3m-url-decode-string x)))
+  (let (method enctype action charset target name)
+    (setq method (case (/ (w3m-char-to-int (car x)) 16)
+		   (0 "get")
+		   (1 "post")
+		   (2 "internal")
+		   (3 "head"))
+	  enctype (case (% (w3m-char-to-int (car x)) 16)
+		    (0 'urlencoded)
+		    (1 'multipart)))
+    (setq x (cdr x))
+    (setq action (w3m-form-mee-attr-unquote x))
+    (setq x (cdr x))
+    (if (member "lang=many" w3m-compile-options)
+	(setq charset (w3m-form-mee-attr-unquote x))
+      (setq charset (case (car x)
+		      (?e "euc-jp")
+		      (?s "shift-jis")
+		      (?n "iso-2022-7bit"))))
+    (setq x (cdr x))
+    (setq target (w3m-form-mee-attr-unquote x)) ; not used.
+    (setq x (cdr x))
+    (setq name (w3m-form-mee-attr-unquote x))   ; not used.
+    (w3m-form-new method action nil (and charset (list charset)) enctype)))
+
+(defun w3m-form-mee-select-value (value)
+  "Decode select form information of w3mmee."
+  (let ((clist (w3m-string-to-char-list (w3m-url-decode-string value)))
+	label val s selected candidates)
+    (while clist
+      (setq s (eq (car clist) (w3m-int-to-char 1))
+	    label nil
+	    val nil)
+      (setq clist (cdr clist))
+      (while (not (eq (car clist) (w3m-int-to-char 0)))
+	(setq label (concat label (char-to-string (car clist))))
+	(setq clist (cdr clist)))
+      (if label
+	  (setq label (decode-coding-string label w3m-output-coding-system)))
+      (setq clist (cdr clist))
+      (while (not (eq (car clist) (w3m-int-to-char 0)))
+	(setq val (concat val (char-to-string (car clist))))
+	(setq clist (cdr clist)))
+      (if val
+	  (setq val (decode-coding-string val w3m-output-coding-system)))
+      (if s (setq selected val))
+      (push (cons val label) candidates)
+      (setq clist (cdr clist)))
+    (cons selected (nreverse candidates))))
+
+(defun w3m-form-parse-and-fontify (&optional reuse-forms)
+  "Parse forms of the half-dumped data in this buffer and fontify them.
+Result form structure is saved to the local variable `w3m-current-forms'.
+If optional REUSE-FORMS is non-nil, reuse it as `w3m-current-form'."
+  (let ((case-fold-search t)
+	tag start end internal-start textareas selects forms maps mapval
+	form)
+    (goto-char (point-min))
+    (while (re-search-forward (if (eq w3m-type 'w3mmee)
+				  (w3m-tag-regexp-of
+				   "_f" "map" "img_alt" "input_alt"
+				   "/input_alt")
+				(w3m-tag-regexp-of
+				   "form_int" "map" "img_alt" "input_alt"
+				   "/input_alt"))
+			      nil t)
+      (setq tag (downcase (match-string 1)))
+      (goto-char (match-end 1))
+      (setq start (match-end 0))
+      (cond
+       ((string= tag (if (eq w3m-type 'w3mmee) "_f" "form_int"))
+	(if (eq w3m-type 'w3mmee)
+	    (w3m-parse-attributes (_x)
+	      (setq forms (nconc forms (list (w3m-form-mee-new _x)))))
+	  (w3m-parse-attributes (action (method :case-ignore)
+					(fid :integer)
+					(accept-charset :case-ignore)
+					(enctype :case-ignore)
+					(charset :case-ignore))
+	    (setq forms
+		  (cons
+		   (cons
+		    fid
+		    (w3m-form-new
+		     (or method "get")
+		     (or action (and w3m-current-url
+				     (string-match w3m-url-components-regexp 
+						   w3m-current-url)
+				     (substring w3m-current-url 0
+						(or (match-beginning 6)
+						    (match-beginning 8)))))
+		     nil
+		     (if accept-charset
+			 (setq accept-charset
+			       (split-string accept-charset ",")))
+		     (if enctype
+			 (intern enctype)
+		       'urlencoded)))
+		   forms)))))
+       ((string= tag "map")
+	(let (candidates)
+	  (w3m-parse-attributes (name)
+	    (while (and (re-search-forward
+			 (w3m-tag-regexp-of "area" "/map") nil t)
+			(not (char-equal
+			      (char-after (match-beginning 1))
+			      ?/)))
+	      (goto-char (match-end 1))
+	      (w3m-parse-attributes (href alt)
+		(when href
+		  (setq candidates (cons (cons href (or alt href))
+					 candidates)))))
+	    (unless maps (setq maps (w3m-form-new "map" ".")))
+	    (when candidates
+	      (w3m-form-put maps
+			    name
+			    (nreverse candidates))))))
+       ((string= tag "img_alt")
+ 	(w3m-parse-attributes (usemap)
+ 	  (re-search-forward (w3m-tag-regexp-of "/img_alt") nil t)
+ 	  (when (or usemap mapval)
+	    (unless maps (setq maps (w3m-form-new "map" ".")))
+	    (unless usemap (setq usemap mapval))
+	    (when mapval (setq mapval nil))
+ 	    (add-text-properties
+ 	     start (match-beginning 0)
+ 	     `(face w3m-form-face
+ 	     w3m-action (w3m-form-input-map ,maps ,usemap))))))
+       ((string= tag "/input_alt")
+	(replace-match ""))
+       ((string= tag "input_alt")
+	(w3m-parse-attributes ((fid :integer)
+			       (type :case-ignore)
+			       (width :integer)
+			       (maxlength :integer)
+			       (hseq :integer)
+			       (selectnumber :integer) ; select
+			       (textareanumber :integer) ; textarea
+			       (size :integer) ; textarea
+			       (rows :integer) ; textarea
+			       (top_mergin :integer) ; textarea
+			       (checked :bool) ; checkbox, radio
+			       no_effect ; map
+			       name value)
+	  (save-excursion
+	    (search-forward "</input_alt>")
+	    (setq end (match-beginning 0)))
+	  (let ((abs-hseq (or (and (null hseq) 0) (abs hseq))))
+	    (setq w3m-max-anchor-sequence 
+		  (max abs-hseq w3m-max-anchor-sequence))
+	    (if (eq w3m-type 'w3mmee)
+		(setq form (nth fid forms))
+	      (setq form (cdr (assq fid forms))))
+	    (when form
+	      (cond
+	       ((and (string= type "hidden")
+		     (string= name "link"))
+		(setq mapval value))
+	       ((or (string= type "submit")
+		    (string= type "image"))
+		(unless (string= no_effect "true")
+		  (w3m-form-make-button
+		   start end
+		   `(w3m-form-field-id
+		     ,(format "fid=%d/type=%s/name=%s" fid type name)
+		     w3m-action (w3m-form-submit ,form ,name ,value)
+		     w3m-submit (w3m-form-submit ,form ,name
+						 (w3m-form-get ,form ,name))
+		     w3m-anchor-sequence ,abs-hseq))))
+	       ((string= type "reset")
+		(w3m-form-make-button
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   w3m-action (w3m-form-reset ,form)
+		   w3m-anchor-sequence ,abs-hseq)))
+	       ((string= type "textarea")
+		(if (eq w3m-type 'w3mmee)
+		    (w3m-form-put form name 
+				  (decode-coding-string
+				   (w3m-url-decode-string value)
+				   w3m-output-coding-system))
+		  (setq textareas (cons (list textareanumber form name)
+					textareas)))
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-textarea ,form ,hseq)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-form-hseq ,hseq
+		   w3m-anchor-sequence ,abs-hseq))
+		(when (> hseq 0)
+		  (add-text-properties start end `(w3m-form-name ,name))))
+	       ((string= type "select")
+		(if (eq w3m-type 'w3mmee)
+		    (w3m-form-put form name
+				  (w3m-form-mee-select-value value))
+		  (setq selects (cons (list selectnumber form name)
+				      selects)))
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-select ,form ,name)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq)))
+	       ((string= type "password")
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-password ,form ,name)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq)))
+	       ((string= type "checkbox")
+		(let ((cvalue (w3m-form-get form name)))
+		  (w3m-form-put form name
+				(if checked
+				    (cons value cvalue)
+				  cvalue)))
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-checkbox ,form ,name ,value)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq)))
+	       ((string= type "radio")
+		;; Radio button input, one name has one value
+		(w3m-form-put form name
+			      (if checked value
+				(w3m-form-get form name)))
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-radio ,form ,name ,value)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq)))
+	       ((string= type "file")
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input-file ,form ,name ,value)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq)))
+	       (t
+		(w3m-form-put form
+			      name
+			      (or value (w3m-form-get form name)))
+		(add-text-properties
+		 start end
+		 `(w3m-form-field-id
+		   ,(format "fid=%d/type=%s/name=%s" fid type name)
+		   face w3m-form-face
+		   w3m-action (w3m-form-input ,form ,name ,type
+					      ,width ,maxlength ,value)
+		   w3m-submit (w3m-form-submit ,form ,name
+					       (w3m-form-get ,form ,name))
+		   w3m-anchor-sequence ,abs-hseq))))))))))
+    ;; Process <internal> tag.
+    (when (search-forward "<internal>" nil t)
+      (setq internal-start (match-beginning 0))
+      (while (and (null reuse-forms)
+		  (re-search-forward "<\\([a-z]+\\)_int" nil t))
+	(cond
+	 ((string= (match-string 1) "select")
+	  (w3m-parse-attributes ((selectnumber :integer))
+	    (let ((selectinfo (cdr (assq selectnumber selects)))
+		  current candidates)
+	      (when selectinfo
+		;; Parse FORM SELECT fields until </SELECT> (or </FORM>)
+		(while (and (re-search-forward
+			     (w3m-tag-regexp-of "option_int" "/select_int")
+			     nil t)
+			    (not (char-equal (char-after (match-beginning 1))
+					     ?/)))
+		  ;; <option_int> is found
+		  (goto-char (match-end 1))
+		  (w3m-parse-attributes ((value :decode-entity)
+					 (label :decode-entity)
+					 (selected :bool))
+		    (push (cons value label) candidates)
+		    (if selected (setq current value))
+		    (skip-chars-forward ">\n")))
+		(setq candidates (nreverse candidates))
+		(w3m-form-put (nth 0 selectinfo)
+			      (nth 1 selectinfo)
+			      (cons (or current ; current value
+					(caar candidates))
+				    candidates))))))
+	 ((string= (match-string 1) "textarea")
+	  (w3m-parse-attributes ((textareanumber :integer))
+	    (forward-char 1) ; skip newline character.
+	    (let ((textareainfo (cdr (assq textareanumber textareas)))
+		  end)
+	      (when textareainfo
+		(setq start (point))
+		(skip-chars-forward "^<")
+		(w3m-form-put (nth 0 textareainfo)
+			      (nth 1 textareainfo)
+			      (w3m-decode-entities-string
+			       (buffer-substring start (point))))))))))
+      (when (search-forward "</internal>" nil t)
+	(delete-region internal-start (match-end 0))))
+    (setq w3m-current-forms (if (eq w3m-type 'w3mmee)
+				forms
+			      (mapcar 'cdr
+				      (sort forms (lambda (x y)
+						    (< (car x)(car y)))))))
+    (w3m-form-resume (or reuse-forms w3m-current-forms))))
 
 (defun w3m-form-replace (string &optional invisible)
   (let* ((start (text-property-any
@@ -559,9 +654,10 @@ return them with the flag."
 		      (make-string (length string) ?.)
 		    (mapconcat 'identity
 			       (split-string
-				(truncate-string string width) "\n")
+				(w3m-truncate-string (or string "")
+						     width) "\n")
 			       "")))
-	    (make-string (- width (string-width string)) ?\ ))
+	    (make-string (max (- width (string-width string)) 0) ?\ ))
     (delete-region (point)
 		   (next-single-property-change (point) 'w3m-action))
     (add-text-properties start (point) prop)
@@ -688,6 +784,8 @@ character."
   (setq w3m-form-input-textarea-keymap (make-sparse-keymap))
   (define-key w3m-form-input-textarea-keymap "\C-c\C-c"
     'w3m-form-input-textarea-set)
+  (define-key w3m-form-input-textarea-keymap "\C-g"
+    'w3m-form-input-textarea-exit)
   (define-key w3m-form-input-textarea-keymap "\C-c\C-q"
     'w3m-form-input-textarea-exit)
   (define-key w3m-form-input-textarea-keymap "\C-c\C-k"
@@ -822,6 +920,12 @@ character."
     'w3m-form-input-select-exit)
   (define-key w3m-form-input-select-keymap "q"
     'w3m-form-input-select-exit)
+  (define-key w3m-form-input-select-keymap "\C-g"
+    'w3m-form-input-select-exit)
+  (define-key w3m-form-input-select-keymap "h" 'backward-char)
+  (define-key w3m-form-input-select-keymap "j" 'next-line)
+  (define-key w3m-form-input-select-keymap "k" 'previous-line)
+  (define-key w3m-form-input-select-keymap "l" 'forward-char)
   (if (featurep 'xemacs)
       (define-key w3m-form-input-select-keymap [(button2)]
 	'w3m-form-input-select-set-mouse)
@@ -934,7 +1038,9 @@ character."
 	(setq value (cdr value))
 	(dolist (candidate value)
 	  (setq pos (point))
-	  (insert (cdr candidate))
+	  (insert (if (zerop (length (cdr candidate)))
+		      " " ; "" -> " "
+		    (cdr candidate)))
 	  (add-text-properties pos (point)
 			       (list 'w3m-form-select-value (car candidate)
 				     'mouse-face w3m-form-mouse-face))
@@ -982,6 +1088,12 @@ character."
     'w3m-form-input-map-exit)
   (define-key w3m-form-input-map-keymap "q"
     'w3m-form-input-map-exit)
+  (define-key w3m-form-input-map-keymap "\C-g"
+    'w3m-form-input-map-exit)  
+  (define-key w3m-form-input-map-keymap "h" 'backward-char)
+  (define-key w3m-form-input-map-keymap "j" 'next-line)
+  (define-key w3m-form-input-map-keymap "k" 'previous-line)
+  (define-key w3m-form-input-map-keymap "l" 'forward-char)  
   (if (featurep 'xemacs)
       (define-key w3m-form-input-map-keymap [(button2)]
 	'w3m-form-input-map-set-mouse)
@@ -1075,7 +1187,9 @@ character."
       (when value
 	(dolist (candidate value)
 	  (setq pos (point))
-	  (insert (cdr candidate))
+	  (insert (if (zerop (length (cdr candidate)))
+		      (car candidate)
+		    (cdr candidate)))
 	  (add-text-properties pos (point)
 			       (list 'w3m-form-map-value (car candidate)
 				     'mouse-face w3m-form-mouse-face))
@@ -1090,13 +1204,19 @@ character."
 (defun w3m-form-submit (form &optional name value)
   (when (and name (not (zerop (length name))))
     (w3m-form-put form name value))
-  (let ((url (or (w3m-form-action form)
-		 (if (string-match "\\?" w3m-current-url)
-		     (substring w3m-current-url 0 (match-beginning 0))
-		   w3m-current-url))))
-    (cond ((eq 'get (w3m-form-method form))
+  (let* ((orig-url w3m-current-url)
+	 (url (or (w3m-form-action form)
+		  (if (string-match "\\?" w3m-current-url)
+		      (substring w3m-current-url 0 (match-beginning 0))
+		    w3m-current-url))))
+    (cond ((and (not (string= url orig-url))
+		(string-match "^https://" orig-url)
+		(string-match "^http://" url)
+		(not (y-or-n-p (format "Send POST data to '%s'?" url))))
+	   (ding))
+	  ((eq 'get (w3m-form-method form))
 	   (w3m-goto-url
-	    (concat url "?" (w3m-form-make-form-data form 'urlencode))))
+	    (concat url "?" (w3m-form-make-form-data form))))
 	  ((eq 'post (w3m-form-method form))
 	   (w3m-goto-url url 'reload nil
 			 (w3m-form-make-form-data form)
