@@ -50,14 +50,20 @@
   (defvar w3m-menubar)
   (defvar w3m-default-coding-system))
 
+(eval-and-compile
+  (if (locate-library "bitmap")
+      (require 'w3m-bitmap)
+    ;; Dummy functions.
+    (defalias 'w3m-create-image 'ignore)
+    (defalias 'w3m-insert-image 'ignore)
+    (defalias 'w3m-remove-image 'ignore)
+    (defalias 'w3m-image-type-available-p 'ignore)
+    (defalias 'w3m-display-graphic-p 'ignore)
+    (defalias 'w3m-display-inline-images-p 'ignore)))
+
 ;; Dummy functions.
-(defalias 'w3m-create-image 'ignore)
-(defalias 'w3m-insert-image 'ignore)
-(defalias 'w3m-image-type-available-p 'ignore)
 (defalias 'w3m-setup-toolbar 'ignore)
 (defalias 'w3m-update-toolbar 'ignore)
-(defalias 'w3m-display-graphic-p 'ignore)
-(defalias 'w3m-display-inline-image-p 'ignore)
 
 ;; Required for old Emacsen.  See the file README for details.
 (eval-and-compile
@@ -128,7 +134,7 @@ variants in this alist.")
     (vscii		. *coding-category-bin*))
   "*Alist of a modern coding-system and a traditional coding-category.")
 
-(let ((id "(generated automatically by Emacs-W3M)")
+(let ((id "(generated automatically by emacs-w3m)")
       to from info-vector document i)
   (dolist (elem w3m-om-coding-system-alist)
     (setq to (car elem)
@@ -309,7 +315,7 @@ empty input."
 							  history
 							  default-value)
 				    activate)
-	"Advised by Emacs-W3M.
+	"Advised by emacs-w3m.
 Allow the optional fourth argument DEFAULT-VALUE which will be used as
 the value to return if the user enters the empty string."
 	(, (if (and (= emacs-major-version 19) (>= emacs-minor-version 29))
@@ -337,6 +343,51 @@ the value to return if the user enters the empty string."
 		       (stringp default-value))
 		  (setq ad-return-value default-value))))))))
 
+(unless (fboundp 'multibyte-string-p)
+  (defalias 'multibyte-string-p 'stringp))
+
+(eval-when-compile
+  ;; Pickup `move-to-column-strictly'.
+  (require 'rect))
+
+(defun move-to-column-force (column)
+  "Move point to column COLUMN rigidly in the current line.
+If COLUMN is within a multi-column character, replace it by
+spaces and tab."
+  (inline (move-to-column-strictly column t)))
+
+(unless (fboundp 'compose-mail)
+  (defun compose-mail (&optional to subject other-headers continue
+				 switch-function yank-action send-actions)
+    "Start composing a mail message to send."
+    (interactive
+     (list nil nil nil current-prefix-arg))
+    (let ((function (get mail-user-agent 'composefunc)))
+      (funcall function to subject other-headers continue
+	       switch-function yank-action send-actions))))
+
+;;; Faces:
+(defvar w3m-om-use-overstrike-to-make-face-bold 'w3m
+  "*If non-nil, use `set-face-bold-p' to make faces bold by overstriking.
+If it is the symbol `w3m', only 'w3m-' prefixed faces will be affected.")
+
+(defadvice custom-declare-face (around set-face-bold-with-overstrike activate)
+  "Advised by emacs-w3m.
+Use `set-face-bold-p' to make faces bold by overstriking.  See also the
+documentation for `w3m-om-use-overstrike-to-make-face-bold'."
+  (if (if (eq w3m-om-use-overstrike-to-make-face-bold 'w3m)
+	  (string-match "\\`w3m-" (symbol-name (ad-get-arg 0)))
+	w3m-om-use-overstrike-to-make-face-bold)
+      (let ((si:custom-set-face-bold (symbol-function 'custom-set-face-bold))
+	    (si:custom-face-bold (symbol-function 'custom-face-bold)))
+	(defalias 'custom-set-face-bold 'set-face-bold-p)
+	(defalias 'custom-face-bold (lambda (face &rest args)
+				      (face-bold-p face)))
+	(unwind-protect
+	    ad-do-it
+	  (fset 'custom-set-face-bold si:custom-set-face-bold)
+	  (fset 'custom-face-bold si:custom-face-bold)))
+    ad-do-it))
 
 ;;; Widget:
 (defun w3m-om-define-missing-widgets ()
