@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 2001 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
-;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
+;; Authors: TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 ;; Keywords: w3m, WWW, hypermedia
 
 ;; This file is a part of emacs-w3m.
@@ -80,7 +80,7 @@
 	      ("4/3420" "$B5\>k8)!&;31h$$(B" "miyagikenyamazoi")
 	      ("6/3510" "$B;37A8)!&B<;3(B" "yamagatakenmurayama" "murayama")
 	      ("6/3520" "$B;37A8)!&CV;r(B" "yamagatakenokitama" "okitama")
-	      ("6/3530" "$B;37A8)!&>1Fb(B" "yamagatakenshonai" "shonai")
+	      ("6/3530" "$B;37A8)!&>1Fb(B" "yamagatakenshonai" "shounai")
 	      ("6/3540" "$B;37A8)!&:G>e(B" "yamagatakenmogami" "mogami")
 	      ("7/3610" "$BJ!Eg8)!&CfDL$j(B" "hukushimakennakadoori" "nakadoori")
 	      ("7/3620" "$BJ!Eg8)!&IMDL$j(B" "hukushimakenhamadoori" "hamadoori")
@@ -128,12 +128,12 @@
 	      ("22/5040" "$B@E2,8)!&@>It(B" "shizuokakenseibu")
 	      ("23/5110" "$B0&CN8)!&@>It(B" "aichikenseibu")
 	      ("23/5120" "$B0&CN8)!&ElIt(B" "aichikentoubu")
-	      ("24/5310" "$B;0=E8)!&KLCfIt(B" "miekenhokuchubu")
+	      ("24/5310" "$B;0=E8)!&KLCfIt(B" "miekenhokuchuubu")
 	      ("24/5320" "$B;0=E8)!&FnIt(B" "miekennanbu")
 	      ("25/6010" "$B<"2l8)!&FnIt(B" "shigakennanbu")
 	      ("25/6020" "$B<"2l8)!&KLIt(B" "shigakenhokubu")
-	      ("26/400" "$B5~ETI\!&KLIt(B" "kyotohuhokubu")
-	      ("26/6100" "$B5~ETI\!&FnIt(B" "kyotohunanbu")
+	      ("26/400" "$B5~ETI\!&KLIt(B" "kyoutohuhokubu")
+	      ("26/6100" "$B5~ETI\!&FnIt(B" "kyoutohunanbu")
 	      ("27/6200" "$BBg:eI\(B" "oosakahu" "oosaka")
 	      ("28/500" "$BJ<8K8)!&KLIt(B" "hyougokenhokubu")
 	      ("28/6300" "$BJ<8K8)!&FnIt(B" "hyougokennanbu")
@@ -143,7 +143,7 @@
 	      ("30/6520" "$BOB2N;38)!&FnIt(B" "wakayamakennanbu")
 	      ("31/6910" "$BD;<h8)!&ElIt(B" "tottorikentoubu")
 	      ("31/6920" "$BD;<h8)!&@>It(B" "tottorikenseibu")
-	      ("32/600" "$BEg:,8)!&1#4t(B" "tottorikenoki")
+	      ("32/600" "$BEg:,8)!&1#4t(B" "shimanekenoki")
 	      ("32/6810" "$BEg:,8)!&ElIt(B" "shimanekentoubu")
 	      ("32/6820" "$BEg:,8)!&@>It(B" "shimanekenseibu")
 	      ("33/6610" "$B2,;38)!&FnIt(B" "okayamakennanbu")
@@ -164,20 +164,20 @@
 	      ("39/7420" "$B9bCN8)!&ElIt(B" "kouchikentoubu")
 	      ("39/7430" "$B9bCN8)!&@>It(B" "kouchikenseibu")
 	      ("40/8210" "$BJ!2,8)!&J!2,(B" "hukuokakenhukuoka" "hukuoka")
-	      ("40/8220" "$BJ!2,8)!&KL6e=#(B" "hukuokakenkitakyushu" "kitakyushu")
+	      ("40/8220" "$BJ!2,8)!&KL6e=#(B" "hukuokakenkitakyushu" "kitakyuushu")
 	      ("40/8230" "$BJ!2,8)!&C^K-(B" "hukuokakenchikuhou" "chikuhou")
 	      ("40/8240" "$BJ!2,8)!&C^8e(B" "hukuokakenchikugo" "chikugo")
 	      ("41/8510" "$B:42l8)!&FnIt(B" "sagakennanbu")
 	      ("41/8520" "$B:42l8)!&KLIt(B" "sagakenhokubu")
 	      ("42/700" "$BD9:j8)!&0m4tBPGO(B"
-	       "nagasakikeniktsushima" "iki" "tsushima" "ikitsushima")
+	       "nagasakikenikitsushima" "iki" "tsushima" "ikitsushima")
 	      ("42/800" "$BD9:j8)!&8^Eg(B" "nagasakikengotou" "gotou")
 	      ("42/8410" "$BD9:j8)!&FnIt(B" "nagasakikennanbu")
 	      ("42/8420" "$BD9:j8)!&KLIt(B" "nagasakikenhokubu")
 	      ("43/8610" "$B7'K\8)!&7'K\(B" "kumamotokenkumamoto" "kumamoto")
 	      ("43/8620" "$B7'K\8)!&0$AI(B" "kumamotokenaso" "aso")
 	      ("43/8630" "$B7'K\8)!&E7Ap02KL(B"
-	       "kumamotokenamakusaashikita" "amakusa" "ashikita")
+	       "kumamotokenamakusaashikita" "amakusa" "ashikita" "amakusaashikita")
 	      ("43/8640" "$B7'K\8)!&5eKa(B" "kumamotokenkuma" "kuma")
 	      ("44/8310" "$BBgJ,8)!&CfIt(B" "ooitakenchuubu")
 	      ("44/8320" "$BBgJ,8)!&KLIt(B" "ooitakenhokubu")
@@ -190,7 +190,7 @@
 	      ("46/8810" "$B</;yEg8)!&;'K`(B" "kagoshimakensatsuma" "satsuma")
 	      ("46/8820" "$B</;yEg8)!&Bg6y(B" "kagoshimakenoosumi" "oosumi")
 	      ("46/900" "$B</;yEg8)!&<o;REg!&205WEg(B"
-	       "kagoshimakentanegashimayakushima" "tanegashima" "yakushima")
+	       "kagoshimakentanegashimayakushima" "tanegashima" "yakushima" "tanegashimayakushima")
 	      ("46/1000" "$B</;yEg8)!&1bH~(B" "kagoshimakenamami" "amami")
 	      ("47/9110" "$B2-Fl8)!&K\EgCfFnIt(B"
 	       "okinawakenhontouchuunanbu" "hontouchuunanbu")
@@ -331,7 +331,9 @@
 (defcustom w3m-weather-filter-functions
   '(w3m-weather-remove-headers
     w3m-weather-remove-footers
-    w3m-weather-insert-title)
+    w3m-weather-expand-anchors
+    w3m-weather-insert-title
+    w3m-weather-insert-seikatu-sisu)
   "Filter functions to remove useless tags."
   :group 'w3m
   :type 'hook)
@@ -407,38 +409,31 @@
   (w3m (format "about://weather/%s" area)))
 
 ;;;###autoload
-(defun w3m-about-weather (url no-decode no-cache &rest args)
-  (let (area furl)
-    (if (and (string-match "^about://weather/" url)
-	     (setq area (substring url (match-end 0))
-		   furl (nth 2 (assoc area w3m-weather-completion-table)))
-	     (w3m-retrieve furl nil no-cache))
-	(w3m-with-work-buffer
-	  (w3m-decode-buffer furl)
-	  (run-hook-with-args 'w3m-weather-filter-functions area furl)
-	  "text/html")
-      (w3m-message "Unknown URL: %s" url)
-      nil)))
+(defun w3m-about-weather (url no-decode no-cache post-data referer handler)
+  (if (string-match "\\`about://weather/" url)
+      (lexical-let* ((url url)
+		     (area (substring url (match-end 0)))
+		     (furl (nth 2 (assoc area w3m-weather-completion-table))))
+	(w3m-process-do
+	    (type (w3m-retrieve furl nil no-cache nil nil handler))
+	  (when type
+	    (w3m-decode-buffer furl)
+	    (run-hook-with-args 'w3m-weather-filter-functions area furl)
+	    "text/html")))
+    (w3m-message "Unknown URL: %s" url)
+    nil))
 
 (defun w3m-weather-remove-headers (&rest args)
   "Remove header of the weather forecast page."
   (goto-char (point-min))
-  (when (search-forward "\
-<TABLE border=\"0\" CELLSPACING=\"1\" CELLPADDING=\"0\" width=\"100%\">
-<tr><td>
-
-<table border=\"0\" CELLSPACING=\"0\" CELLPADDING=\"0\" width=\"100%\">
-<tr><td bgcolor=\"#dcdcdc\"><b>$B:#F|!&L@F|$NE75$(B</b></td>" nil t)
+  (when (search-forward "<!--- TITLE_TABLE_1 --->" nil t)
     (delete-region (point-min) (match-beginning 0))))
 
 (defun w3m-weather-remove-footers (&rest args)
   "Remove footer of the weather forecast page."
   (goto-char (point-max))
-  (when (search-backward "\
-<table border=0 cellpadding=2 cellspacing=5 width=\"100%\">
-<tr bgcolor=\"#dcdcdc\">
-<td colspan=3><b>$B%l%8%c!<E75$(B</b></td></tr>" nil t)
-    (delete-region (point) (point-max))))
+  (when (search-backward "<!--- LEISURE_LINK --->" nil t)
+    (delete-region (match-beginning 0) (point-max))))
 
 (defun w3m-weather-insert-title (area url &rest args)
   "Insert title."
@@ -452,6 +447,63 @@
   (goto-char (point-max))
   (insert "</body>"))
 
+(defun w3m-weather-expand-anchors (area url &rest args)
+  ;; FIXME: $BE75$M=Js%Z!<%8$K4^$^$l$F$$$kAjBP%j%s%/$r@dBP%j%s%/$K=q$-49(B
+  ;; $B$($k$?$a$N4X?t!%$3$l$i$NAjBP%j%s%/$r0BA4$K<h$j07$&$?$a$K$O!$(Bbase
+  ;; URL $B$rJV$;$k$h$&$K!$(Babout:// $B$N9=B$$r=q$-D>$9I,MW$,$"$k$H9M$($i$l(B
+  ;; $B$k$,!$$H$j$"$($:8e2s$7!%(B
+  (goto-char (point-min))
+  (while (re-search-forward
+	  (eval-when-compile
+	    (concat "<a[ \t\r\f\n]+href=" w3m-html-string-regexp))
+	  nil t)
+    (replace-match (format
+		    "<a href=\"%s\""
+		    (w3m-expand-url (w3m-remove-redundant-spaces
+				     (or (match-string-no-properties 2)
+					 (match-string-no-properties 3)
+					 (match-string-no-properties 1)))
+				    url)))))
+
+(defun w3m-weather-get-seikatu-sisu (url &rest args)
+  (with-temp-buffer
+    (w3m-retrieve url)
+    (w3m-decode-buffer url)
+    (goto-char (point-min))
+    (let (sisu-list)
+      (when (search-forward "<td>$B;X!!?t(B</td>" nil t)
+	(while (re-search-forward "<td nowrap bgcolor=\"#CCCCFF\">$B!!(B*\\([^$B!!(B\n]+\\)$B!!(B*</td>" nil t)
+	  (let ((name-sisu (match-string 1))
+		sisu)
+	    (dotimes (i 2)
+	      (forward-line 1)
+	      (when (looking-at "<td bgcolor=\"#EEEEEE\">\\([^\n]+\\)</td>")
+		(push (list name-sisu (match-string 1)) sisu)))
+	    (push sisu sisu-list)))
+	(nreverse sisu-list)))))
+
+(defun w3m-weather-insert-seikatu-sisu (&rest args)
+  (goto-char (point-min))
+  (when (re-search-forward "<a href=\"\\([^\"]+\\)\">$B@83h;X?t(B</a>" nil t)
+    (let ((sisu-list (w3m-weather-get-seikatu-sisu (match-string 1)))
+	  (case-fold-search t)
+	  sisu-name)
+      (goto-char (point-min))
+      (when (search-forward "</table>\n<!--- /WEATHER_TABLE_1 --->\n" nil t)
+	(goto-char (match-beginning 0))
+	(dolist (seikatu-sisu sisu-list)
+	  (insert "<tr>\n")
+	  (dolist (sisu (reverse seikatu-sisu))
+	    (insert "<td width=\"50%\"><table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td nowrap bgcolor=\"#CCCCFF\" width=\"100\" valign=\"top\">")
+	    (insert (format "%s</td>\n" (car sisu)))
+	    (insert (format "<td bgcolor=\"#eeeeee\" align=\"center\">%s</td>\n</tr>\n</table>\n</td>\n" (cadr sisu))))
+	  (insert "</tr>\n"))
+	(goto-char (point-min))
+	(when (search-forward "<!--- EXPONENT_LINK --->" nil t)
+	  (let ((bg (match-beginning 0)))
+	    (when (search-forward "<!--- /EXPONENT_LINK --->" nil t)
+	      (delete-region bg (match-end 0)))))))))
 
 (provide 'w3m-weather)
+
 ;;; w3m-weather.el ends here.
