@@ -1,6 +1,6 @@
 ;;; w3m-xmas.el --- The stuffs to use emacs-w3m on XEmacs
 
-;; Copyright (C) 2001,2002 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
+;; Copyright (C) 2001, 2002 TSUCHIYA Masatoshi <tsuchiya@namazu.org>
 
 ;; Authors: Yuuichi Teranishi  <teranisi@gohome.org>,
 ;;          TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
@@ -35,12 +35,14 @@
 
 ;;; Code:
 
-(require 'w3m-macro)
+(require 'w3m-util)
+(require 'w3m-proc)
+(require 'w3m-image)
 
 ;; Functions and variables which should be defined in the other module
 ;; at run-time.
 (eval-when-compile
-  (defvar w3m-current-image-status)
+  (defvar w3m-display-inline-images)
   (defvar w3m-current-url)
   (defvar w3m-current-title)
   (defvar w3m-default-coding-system)
@@ -52,10 +54,16 @@
   (defvar w3m-toolbar-buttons)
   (defvar w3m-use-header-line)
   (defvar w3m-work-buffer-name)
+  (defvar w3m-use-tab-menubar)
+  (autoload 'w3m-setup-tab-menu "w3m-tabmenu")
   (autoload 'update-tab-in-gutter "gutter-items")
   (autoload 'w3m-image-type "w3m")
   (autoload 'w3m-retrieve "w3m"))
 
+;; Dummies to shut some XEmacs variants up.
+(eval-when-compile
+  (autoload 'unicode-to-char "XEmacs-21.5-b6_and_later"))
+
 (require 'path-util)
 (require 'poe)
 (require 'poem)
@@ -66,9 +74,30 @@
 				      'find-coding-system
 				    'ignore))
 
-(defalias 'w3m-make-ccl-coding-system (if (fboundp 'make-ccl-coding-system)
-					  'make-ccl-coding-system
-					'ignore))
+;; Under XEmacs 21.5-b6 and later, `make-ccl-coding-system' will
+;; signal an error if the coding-system has already been defined.
+;; To make w3m.elc reloadable, we'll define the function as follows:
+;;
+;;(defun w3m-make-ccl-coding-system (coding-system args...)
+;;  (or (find-coding-system coding-system)
+;;      (make-ccl-coding-system coding-system args...)))
+(eval-when-compile
+  (defmacro w3m-xmas-define-w3m-make-ccl-coding-system ()
+    "Make the source form for the function `w3m-make-ccl-coding-system'."
+    (if (and (fboundp 'make-ccl-coding-system)
+	     (fboundp 'find-coding-system))
+	`(defun w3m-make-ccl-coding-system (coding-system mnemonic docstring
+							  decoder encoder)
+	   ,(concat (documentation 'make-ccl-coding-system)
+		    "\n\n\
+NOTE: This function is slightly modified from `make-ccl-coding-system'
+      to be recallable for the existing coding-systems without errors.")
+	   (or (find-coding-system coding-system)
+	       (,(symbol-function 'make-ccl-coding-system)
+		coding-system mnemonic docstring decoder encoder)))
+      '(defalias 'w3m-make-ccl-coding-system 'ignore))))
+
+(w3m-xmas-define-w3m-make-ccl-coding-system)
 
 (unless (fboundp 'coding-system-category)
   (defalias 'coding-system-category 'ignore))
@@ -76,6 +105,12 @@
 (unless (fboundp 'coding-system-list)
   (defalias 'coding-system-list 'ignore))
 
+;; If pccl.elc has been mis-compiled for XEmacs with MULE, the macro
+;; `define-ccl-program' wouldn't be an empty macro because of advice.
+(when (and (not (featurep 'mule))
+	   (featurep 'advice))
+  (ad-unadvise 'define-ccl-program))
+
 (unless (fboundp 'define-ccl-program)
   (defmacro define-ccl-program (&rest args)))
 
@@ -103,16 +138,22 @@ PRIORITY-LIST is a list of coding systems ordered by priority."
 	(car codesys)
       codesys)))
 
+(when (and (not (fboundp 'w3m-ucs-to-char))
+	   (fboundp 'unicode-to-char)
+	   (subrp (symbol-function 'unicode-to-char)))
+  (defun w3m-ucs-to-char (codepoint)
+    (unicode-to-char codepoint)))
+
 ;;; Handle images:
 
 ;; Function which returns non-nil when the current display device can
 ;; show images inline.
 (defalias 'w3m-display-graphic-p 'device-on-window-system-p)
 
-(defun w3m-display-inline-image-p ()
+(defun w3m-display-inline-images-p ()
   "Returns non-nil when images can be displayed under the present
 circumstances."
-  (and w3m-current-image-status (device-on-window-system-p)))
+  (and w3m-display-inline-images (device-on-window-system-p)))
 
 (defvar w3m-should-unoptimize-animated-gifs
   (and (= emacs-major-version 21)
@@ -255,33 +296,93 @@ new glyph image.  See also the documentation for the variable
       ;; Use a cached glyph.
       (cdr cache))))
 
-(defun w3m-create-image (url &optional no-cache referer)
+(defsubst w3m-make-glyph (data type)
+  (or (and (eq type 'xbm)
+	   (let (width height content)
+	     (with-temp-buffer
+	       (insert data)
+	       (goto-char (point-min))
+	       (if (re-search-forward "width[ \t]+\\([0-9]+\\)")
+		   (setq width (string-to-int (match-string 1))))
+	       (if (re-search-forward "height[ \t]+\\([0-9]+\\)")
+		   (setq height (string-to-int (match-string 1))))
+	       (while (re-search-forward "0x\\(..\\)" nil t)
+		 (setq content (cons
+				(string-to-int
+				 (match-string 1) 16) content)))
+	       (setq content (concat (nreverse content))))
+	     (make-glyph (vector 'xbm :data (list width height content)))))
+      (make-glyph (vector type :data data))))
+
+(defun w3m-create-image (url &optional no-cache referer size handler)
   "Retrieve data from URL and create an image object.
 If optional argument NO-CACHE is non-nil, cache is not used.
-If second optional argument REFERER is non-nil, it is used as Referer: field."
-  (let ((type (condition-case err
-		  (w3m-retrieve url 'raw no-cache nil referer)
-		(error
-		 (message "While retrieving %s: %s" url err)
-		 nil))))
-    (when (w3m-image-type-available-p (setq type (w3m-image-type type)))
-      (let ((data (w3m-with-work-buffer (buffer-string))))
-	(or (and (eq type 'gif)
-		 (or w3m-should-unoptimize-animated-gifs
-		     w3m-should-convert-interlaced-gifs)
-		 w3m-gifsicle-program
-		 (w3m-fix-gif url data no-cache))
-	    (make-glyph (vector type :data data)))))))
+If second optional argument REFERER is non-nil, it is used as Referer: field.
+Third optional argument SIZE is currently ignored."
+  (if (not handler)
+      (w3m-process-with-wait-handler
+	(w3m-create-image url no-cache referer handler))
+    (lexical-let ((url url)
+		  (set-size size)
+		  (no-cache no-cache)
+		  size)
+      (w3m-process-do-with-temp-buffer
+	  (type (condition-case err
+		    (w3m-retrieve url 'raw no-cache nil referer handler)
+		  (error (message "While retrieving %s: %s" url err) nil)))
+	(let ((data (buffer-string))
+	      glyph)
+	  (setq glyph
+		(when (w3m-image-type-available-p (setq type
+							(w3m-image-type type)))
+		  (or (and (eq type 'gif)
+			   (or w3m-should-unoptimize-animated-gifs
+			       w3m-should-convert-interlaced-gifs)
+			   w3m-gifsicle-program
+			   (w3m-fix-gif url data no-cache))
+		      (w3m-make-glyph data type))))
+	  (if (and w3m-resize-images set-size)
+	      (progn
+		(setq size (cons (glyph-width glyph)
+				 (glyph-height glyph)))
+		(if (and (null (car set-size)) (cdr set-size))
+		    (setcar set-size
+			    (/ (* (car size) (cdr set-size))
+			       (cdr size))))
+		(if (and (null (cdr set-size)) (car set-size))
+		    (setcdr set-size
+			    (/ (* (cdr size) (car set-size))
+			       (car size))))
+		(if (or (not (eq (car size)
+				 (car set-size)))  ; width is different
+			(not (eq (cdr size)
+				 (cdr set-size)))) ; height is different
+		    (lexical-let ((type type))
+		      (w3m-process-do
+			  (resized (w3m-resize-image
+				    data
+				    (car set-size)(cdr set-size)
+				    handler))
+			(w3m-make-glyph resized type)))
+		  glyph))
+	    glyph))))))
 
-(defun w3m-insert-image (beg end image)
+(defun w3m-insert-image (beg end image &rest args)
   "Display image on the current buffer.
 Buffer string between BEG and END are replaced with IMAGE."
   (let (extent glyphs)
     (while (setq extent (extent-at beg nil 'w3m-xmas-icon extent 'at))
-      (push (extent-end-glyph extent) glyphs))
-    (set-extent-properties (make-extent beg end)
-			   (list 'invisible t 'w3m-xmas-icon t
-				 'end-glyph image))
+      (push (extent-end-glyph extent) glyphs)
+      (set-extent-end-glyph extent nil))
+    (push image glyphs)
+    ;; Image on the right is displayed.
+    (when (extent-at end nil 'invisible nil 'at)
+      (setq end (next-single-property-change end 'invisible))
+      (while (setq extent (extent-at end nil 'w3m-xmas-icon extent 'at))
+	(push (extent-end-glyph extent) glyphs)
+	(set-extent-end-glyph extent nil)))
+    (set-extent-properties (make-extent beg end) (list 'w3m-xmas-icon t
+						       'invisible t))
     (while glyphs
       (set-extent-properties (make-extent end end)
 			     (list 'w3m-xmas-icon t
@@ -289,15 +390,13 @@ Buffer string between BEG and END are replaced with IMAGE."
 
 (defun w3m-remove-image (beg end)
   "Remove an image which is inserted between BEG and END."
-  (let (extent)
+  (let (extent extents)
     (while (setq extent (extent-at beg nil 'w3m-xmas-icon extent 'at))
-      (if (extent-end-glyph extent)
-	  (set-extent-end-glyph extent nil))
-      (set-extent-property extent 'invisible nil))
+      (setq extents (push extent extents)))
     (while (setq extent (extent-at end nil 'w3m-xmas-icon extent 'at))
-      (if (extent-end-glyph extent)
-	  (set-extent-end-glyph extent nil))
-      (set-extent-property extent 'invisible nil))))
+      (setq extents (push extent extents)))
+    (dolist (extent extents)
+      (delete-extent extent))))
 
 (defun w3m-image-type-available-p (image-type)
   "Return non-nil if an image with IMAGE-TYPE can be displayed inline."
@@ -346,6 +445,7 @@ Buffer string between BEG and END are replaced with IMAGE."
   (when (and (featurep 'menubar)
 	     current-menubar
 	     (not (assoc (car w3m-menubar) current-menubar)))
+    (when w3m-use-tab-menubar (w3m-setup-tab-menu))
     (set-buffer-menubar (cons w3m-menubar current-menubar))))
 
 ;;; Widget:
@@ -402,35 +502,10 @@ as the value."
 
 (eval-after-load "wid-edit" '(w3m-xmas-define-missing-widgets))
 
-;;; Header line (emulating Emacs 21):
+;;; Header line:
 (defvar w3m-header-line-map (make-sparse-keymap))
 (define-key w3m-header-line-map 'button2 'w3m-goto-url)
 
-(defun w3m-setup-header-line ()
-  "Setup header line (emulating Emacs 21)."
-  (when (and w3m-use-header-line w3m-current-url
-	     (eq 'w3m-mode major-mode))
-    (goto-char (point-min))
-    (insert "Location: ")
-    (put-text-property (point-min) (point)
-		       'face 'w3m-header-line-location-title-face)
-    (let ((start (point))
-	  (help "button2 prompts to input URL"))
-      (insert w3m-current-url)
-      (add-text-properties start (point)
-			   (list 'face
-				 'w3m-header-line-location-content-face
-				 'mouse-face 'highlight
-				 'keymap w3m-header-line-map
-				 'help-echo help
-				 'balloon-help help))
-      (setq start (point))
-      (insert-char ?\  (max 0 (- (window-width) (current-column) 1)))
-      (put-text-property start (point)
-			 'face 'w3m-header-line-location-content-face)
-      (unless (eolp)
-	(insert "\n")))))
-
 ;;; Gutter:
 (defcustom w3m-xmas-show-current-title-in-buffer-tab
   (and (boundp 'gutter-buffers-tab-enabled)
@@ -517,6 +592,11 @@ title contains non-ascii characters, show a url name by default."
   (when (symbol-value 'gutter-buffers-tab-enabled)
     (add-hook 'w3m-display-hook 'w3m-xmas-update-tab-in-gutter)))
 
+;;; Miscellaneous:
+(if (featurep 'mule)
+    (defalias 'multibyte-string-p 'stringp)
+  (defalias 'multibyte-string-p 'ignore))
+
 (provide 'w3m-xmas)
 
 ;;; w3m-xmas.el ends here
